
# q-expansion of newform 240.6.a.j, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 240
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [9], [-25], [100], [136], [82], [358], [-796], [-488], [7466], [-2728], [7794], [18234], [2444], [2200], [10122], [6776], [23398], [9676], [-13728], [-27390], [93288], [23276], [102354], [-49502], [107682], [214676], [-50892], [-17242], [-40338], [-243828], [-303336], [224486], [-239628], [387754], [-532184], [46162], [-185188], [-456696], [-202526], [7808], [194366], [-383560], [-718262], [342218], [-311336], [-684500], [-32708], [1203012], [90022], [-1611090], [898296], [-343102], [-20400], [-117522], [1756152], [548170], [-1964448], [1425266], [-1520014], [-1103948], [1241258], [772772], [-1314976], [693986], [1792194], [-1053780], [2927002], [1269068], [302790], [2902918], [-1755816], [1208092], [3924490], [3340340], [3100200], [1347498], [1791226], [-5445030], [-3643526], [-3287352], [-5509050], [372368], [3182714], [5206520], [-5324100], [1285626], [-6071558], [-5591406], [-6058812], [-2666356], [-1765784], [1273388], [-6079968], [9536548], [-1974576], [2477546], [-4010254], [-10048636], [4524742], [10656724], [-8095454], [5559028], [4100050], [5760764], [11342978], [-6676524], [10640094], [-2230824], [12884362], [-3033420], [6545202], [8453598], [13864324], [-13528096], [-12029214], [5923932], [1262688], [-18710846], [-952872], [5167710], [3341946], [-19424758], [15626244], [21964324], [-6160694], [-20983210], [10247872], [-15974612], [-20354694], [11998644], [24134592], [1151496], [11933042], [13891698], [-10696958], [-10582662], [-6237812], [4592658], [-13974], [-20597084], [-6288190], [-25167316], [-28127028], [8419030], [-18955096], [-40236670], [-1798002], [-30541868], [26644128], [-21093422], [13850730], [-856484], [-30311672], [19632684], [-2842496], [-42068576], [-35009350], [-13663142], [30024386], [-10246508], [49693734], [-36055836], [-46494016], [-4394570], [2366016], [14839840], [-48890934], [-13468606], [59438170], [-5849656], [4008670], [22594040], [-43120270], [25549240], [-18828654], [53572148], [-39386630], [-49582564], [39073534], [25236388], [12374472], [-5106334], [-13385146], [38297768], [35789962], [52972370], [55668844], [2427770], [40277080], [53953322], [-37879332], [45307420], [-30640718], [4759084], [63973702], [6011826], [-33213726], [-39763682], [-84535936], [54094338], [62260952], [61102682], [-36592686], [7858200], [-33047246], [-72848776], [54632692], [-85532382], [62044396], [76794442], [-93938918], [-26615612], [-67599588], [94412032], [-20945862], [93173420], [-90930662], [15032256], [46778546], [54908870], [-101974360], [-132095422], [76502388], [104334780], [125488150], [-921914], [51086912], [35715388], [-135545512], [-112327214], [26360892], [14762560], [-49385230], [54358260], [10492416], [68550450], [-131815590], [-37274360], [-137316720], [52506724], [-177516772], [139871324], [53461366], [110807302], [-67147840], [86927804], [-16668400], [-114121252], [38800304], [28008762], [115171010], [-71312232], [-30533830], [-86977454], [30640728], [-51486322], [38395524], [123432282], [171890082], [22348892], [-148389820], [154135542], [-198420694], [201413134], [-6217996], [79092818], [97151466], [-10035036], [41039978], [-36074770], [45836228], [33612498], [151734520], [92207834], [64217476], [229906108], [-76730698], [-160462390], [57022584], [196815120], [55644984], [-166257088], [30509542], [-152465668], [-101136536], [-75008590], [-83122182], [-36891624], [-121026078], [-201392206], [250303212], [225432318], [200145672], [-98934422], [124135266], [-235821560], [-260130534], [16081432], [-304061196], [187781586], [-121499262], [97374160], [-140126260], [-70054644], [28396018], [126197076], [-87023714], [344732064], [66083258], [-79770032], [-107737710], [-10676886], [-372253180], [-157758208], [-245314502], [-358206656], [294207432], [333198778], [90836162], [-183307100], [-229356382], [21933402], [-237787996], [-84528418], [214325058], [-147118372], [-422807876], [64820648], [-53760630], [-91761682], [148918802], [-100664344], [-224905796], [245198132], [211366404], [-283269682], [-295434666], [92636458], [162707532], [188740538], [66571374], [132049058], [181595608], [-83457694], [-196600640], [67855798], [-157471588], [412028040], [-221589126], [116233484], [-427458918], [208665722], [242264716], [-279106690], [-290859586], [-282002064], [236073976], [257992286], [-424987384], [215644202], [-38274838], [74742968], [70343160], [-87417588], [525791338], [215691282], [421546948], [36064442], [-318500928], [441302404], [-260125480], [272478498], [279374904], [-43679878], [-1375200], [-625177024], [92018066], [-69073494], [14475194], [702966658], [63728870], [83971428], [-290231850], [-173031540], [-70210616], [-588698344], [520812498], [481302444], [-352179488], [-393195902], [-129504342], [-209613528], [-444301540], [413630648], [-731493238], [173385856], [541724874], [-691334852], [211165626], [-184466178], [-664640898], [387576860], [341777622], [-198353454], [230814096], [-195672998], [-709111014], [538862300], [-633713832], [-64196414], [-144890886], [-343285516], [-490574236], [-633510782], [567445482], [244517920], [70601252], [-606259410], [515356440], [-601395390], [178090378], [-526324376], [151773616], [-801691230], [585142530], [49444908], [-551574246], [-151066628], [649173752]]
